package cn.com.ctop.job.controller;

import cn.com.ctop.job.bytedance.entity.WechatAttendance;
import cn.com.ctop.job.bytedance.service.WechatService;
import cn.com.ctop.job.bytedance.utils.Check;
import cn.com.ctop.job.bytedance.utils.ExportExcelUtils;
import cn.com.ctop.job.bytedance.utils.HttpUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("/bytedanceController")
@Slf4j
public class BytedanceController {

    @Autowired

    private WechatService wechatService;


    @GetMapping(value = "/test")
    public String test(Long accountId) {
        System.out.println("ok");
        return "ok";
    }


    //更新人员和部门
    @GetMapping("getUser")
    public String getUser() {
        long startTime = System.currentTimeMillis();
        wechatService.getUserList();
        String txt = "执行更新人员和部门完成,用时：" + (System.currentTimeMillis() - startTime) / 1000 + " s ";
        System.out.println(txt);
        return txt;
    }


    //日常考勤数据
    @GetMapping("getCheckinData")
    public String getCheckinData(String start, String end) {
        if (Check.isNull(start) || Check.isNull(end)) {
            return "时间参数不能为空";
        }
        String txt = null;
        try {
            Long startTime = System.currentTimeMillis();
            wechatService.getCheckinData(start + " 00:00:00", end + " 23:59:59");
            Thread.sleep(5000L);
            wechatService.getNoListDate(start + " 00:00:00", end + " 23:59:59", "0");

            txt = "考勤执行完成,用时：" + (System.currentTimeMillis() - startTime) / 1000 + " s ";
            System.out.println(txt);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return txt;
    }


    //异常考勤数据
    @GetMapping("getNoList")
    public String getNoList(String start, String end) {
        long startTime = System.currentTimeMillis();
        wechatService.getNoListDate(start + " 00:00:00", end + " 23:59:59", "0");
        String txt = "异常考勤执行完成,用时：" + (System.currentTimeMillis() - startTime) / 1000 + " s ";
        return txt;
    }


    /**
     * 考勤记录 导出数据
     */
    @GetMapping("/exportData")
    public void getExcelReport(HttpServletResponse response, String start, String end, String departName, Long departId, String userName, String userId) {
        if (Check.isNull(start) || Check.isNull(end)) {
            return;
        }
        Map<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("startTime", start + " 00:00:00");
        paramsMap.put("stopTime", end + " 23:59:59");
        paramsMap.put("departName", departName);
        paramsMap.put("departId", departId);
        paramsMap.put("userName", userName);
        paramsMap.put("userId", userId);
        List<WechatAttendance> excelData = wechatService.queryAttendanceData(paramsMap);
        List<WechatAttendance> overtimeData = wechatService.queryOverTimeData(paramsMap);
        try {
            List<List<Object>> excelList = new ArrayList<>();
            List<List<Object>> excelYRList = new ArrayList<>();
            List<List<Object>> overtimeExcelList = new ArrayList<>();
            List<List<Object>> overtimeYRExcelList = new ArrayList<>();
            if (!excelData.isEmpty()) {
                excelData.forEach(data -> {
                    List<Object> temp = new ArrayList<>();
                    temp.add(data.getUserName());
                    temp.add(data.getDepartName());
                    temp.add(data.getCheckinDate());
                    temp.add(data.getCheckinType());
                    temp.add(data.getExceptionType());
                    temp.add(data.getCheckinTime());
                    temp.add(data.getType());
                    temp.add(data.getStateDuration());
                    if (!Check.isNull(data.getStatus())) {
                        temp.add("1".equals(data.getStatus()) ? "审批中" : "已通过");
                    } else {
                        temp.add("");
                    }
                    excelList.add(temp);
                });
            }
            if (!overtimeData.isEmpty()) {
                overtimeData.forEach(data -> {
                    List<Object> temp = new ArrayList<>();
                    temp.add(data.getUserName());
                    temp.add(data.getDepartName());
                    temp.add(data.getCheckinDate());
                    temp.add(data.getOvertime());
                    overtimeExcelList.add(temp);
                });
            }
            OutputStream os = response.getOutputStream();
            ExportExcelUtils eeu = new ExportExcelUtils();
            XSSFWorkbook workbook = new XSSFWorkbook();
            String[] titles = {"姓名", "部门", "考勤日期", "考勤类型", "考勤状态", "考勤时间", "审核状态", "状态时长", "审批状态"};
            String[] overtimeTitles = {"姓名", "部门", "加班日期", "加班时长"};
            eeu.exportExcelForAttendance(workbook, 0, "汇创异常考勤", titles, excelList);
            eeu.exportExcel(workbook, 1, "汇创加班考勤", overtimeTitles, overtimeExcelList);
            HttpUtils.setResponseHeader(response, start + "~" + end + " clockIn.xlsx");
            workbook.write(os);
            os.flush();
            os.close();
        } catch (IOException e) {
            log.error(e.getMessage());
        }
    }

}