package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.entity.CtopOauthToken;
import cn.com.ctop.job.bytedance.service.CommonService;
import cn.com.ctop.job.bytedance.service.TokenService;
import cn.com.ctop.job.bytedance.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 头条预警
 *
 * @author yumeng
 */
@Component
public class ByteDanceHourlyAccountWarningLoadJob {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private CommonService commonService;

    static ExecutorService executorService = Executors.newFixedThreadPool(3);

    /**
     * 头条时段消耗预警
     */
    @XxlJob("byteDanceHourlyAccountWarning")
    public void execute() throws Exception {
        try {
            XxlJobHelper.log("开始时段消耗预警定时任务");
            long startTime = System.currentTimeMillis();
            List<CtopOauthToken> tokens = tokenService.getAllBytedanceTokens();
            if (null == tokens || tokens.isEmpty()) {
                XxlJobHelper.log("定时获取头条数据异常：未获取到可用的token");
                XxlJobHelper.handleFail();
                return;
            }
            String statDate = DateUtils.formatDate();
            String anotherDay = DateUtils.getAnotherDay("yyyy-MM-dd", statDate, 1);
            tokens.forEach(token -> executorService.submit(() -> commonService.byteDanceAccountWarning(token.getAccountId(), statDate, anotherDay)));
            XxlJobHelper.log("时段消耗预警定时任务执行完毕，耗时：{} ms", System.currentTimeMillis() - startTime);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
