package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.entity.CtopOauthToken;
import cn.com.ctop.job.bytedance.service.ReportDataService;
import cn.com.ctop.job.bytedance.service.TokenService;
import cn.com.ctop.job.bytedance.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @author 宋英豪
 * 头条广告主日报数据获取任务
 * 第二天凌晨跑前一天的数据
 */
@Component
public class BytedanceAdvertiserDailyReportLoadJob {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private ReportDataService reportService;

    static ExecutorService executorService = Executors.newFixedThreadPool(3);
    ;

    /**
     * 头条广告主日报
     */
    @XxlJob("bytedanceAdvertiserDailyReport")
    public void execute() throws Exception {
        XxlJobHelper.log("头条广告主日报获取任务执行开始");
        Date getDate = DateUtils.addDay(new Date(), -1);
        //1:查询当日数据
        List<CtopOauthToken> tokens = tokenService.getAllBytedanceTokens();
        if (null == tokens || tokens.isEmpty()) {
            XxlJobHelper.log("定时获取头条数据异常：未获取到可用的token");
            XxlJobHelper.handleFail();
            return;
        }
        CountDownLatch countDownLatch = new CountDownLatch(tokens.size());
        tokens.forEach(token -> executorService.submit(() -> {
            try {
                reportService.getAdvertiserReport(token, getDate, getDate, "STAT_TIME_GRANULARITY_DAILY");
            } catch (Exception e) {
                XxlJobHelper.log(e);
            } finally {
                countDownLatch.countDown();
            }
        }));
        countDownLatch.await();
        XxlJobHelper.log("头条广告主日报获取任务执行结束");
    }
}
