package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.job.bytedance.entity.CtopOauthToken;
import cn.com.ctop.job.bytedance.service.ReportDataService;
import cn.com.ctop.job.bytedance.service.TokenService;
import cn.com.ctop.job.bytedance.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @author 宋英豪
 */
@Component
public class BytedanceAdvertiserHourlyReportLoadJob {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private ReportDataService reportService;

    static ExecutorService executorService = Executors.newFixedThreadPool(4);

    /**
     * 头条广告主时报
     */
    @XxlJob("bytedanceAdvertiserHourlyReport")
    public void execute() throws Exception {
        try {
            XxlJobHelper.log("头条时报数据获取开始");
            Date getDate = new Date();
            String hour = DateUtils.getNowHour(getDate);
            //1:查询当日数据
            List<CtopOauthToken> tokens = tokenService.getAllBytedanceTokens();
            if (null == tokens || tokens.isEmpty()) {
                XxlJobHelper.log("定时获取头条小时数据异常：未获取到可用的token");
                XxlJobHelper.handleFail();
                return;
            }
            tokens.forEach(token -> executorService.submit(() -> {
                if ("00".equals(hour)) {
                    Date finalGetDate = DateUtils.addDay(getDate, -1);
                    //获取广告主信息数据
                    reportService.getAdvertiserReport(token, finalGetDate, finalGetDate, "STAT_TIME_GRANULARITY_HOURLY");
                }
                reportService.getAdvertiserReport(token, getDate, getDate, "STAT_TIME_GRANULARITY_HOURLY");
            }));
            XxlJobHelper.log("头条时报数据获取结束");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
