package cn.com.ctop.job.bytedance.service.impl;

import cn.com.ctop.job.bytedance.entity.Project;
import cn.com.ctop.job.bytedance.entity.RuleAccountTemplate;
import cn.com.ctop.job.bytedance.entity.UserAllocation;
import cn.com.ctop.job.bytedance.mapper.AccountInfoMapper;
import cn.com.ctop.job.bytedance.mapper.CommonMapper;
import cn.com.ctop.job.bytedance.service.CommonService;
import cn.com.ctop.job.bytedance.service.ISendMessageService;
import cn.com.ctop.job.bytedance.utils.Check;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

@Slf4j
@Service
public class CommonServiceImpl implements CommonService {
    @Autowired
    private CommonMapper commonMapper;

    @Autowired
    private AccountInfoMapper accountInfoMapper;

    @Autowired
    private ISendMessageService sendMessage;

    @Override
    public List<RuleAccountTemplate> getRuleAccountTemplateByStatus(int status) {
        return commonMapper.getRuleAccountTemplateByStatus(status);
    }

    /**
     * 头条账户 出价预警
     *
     * @param accountId
     */
    @Override
    public void byteDanceAccountWarning(Long accountId, String statDate, String anotherDay) {
        try {
            Integer maxHour = commonMapper.selectMaxHour(accountId, statDate);
            if (Check.isNull(maxHour)) {
                return;
            }
            UserAllocation userAllocation = accountInfoMapper.getUserAllocationByAccountId(accountId);
            if (Check.isNull(userAllocation)) {
                return;
            }
            Project project = accountInfoMapper.getProjectById(userAllocation.getProjectId());
            if (userAllocation.getAccountStatus() == 1) {
                return;
            }
            BigDecimal warningAmount = userAllocation.getWarningAmount();
            BigDecimal warningProportion = userAllocation.getWarningProportion();
            BigDecimal nowCost = commonMapper.getCost(accountId, maxHour, statDate);
            BigDecimal cost = nowCost.setScale(2, RoundingMode.HALF_UP);
            if (maxHour != 0) {
                if (!Check.isNull(nowCost) && nowCost.compareTo(new BigDecimal(0)) != 0) {
                    BigDecimal lastHourCost = commonMapper.getCost(accountId, maxHour - 1, statDate);
                    if (!Check.isNull(lastHourCost) && lastHourCost.compareTo(new BigDecimal(0)) != 0) {
                        BigDecimal subtract = nowCost.subtract(lastHourCost);
                        BigDecimal divide = nowCost.divide(lastHourCost, 2, RoundingMode.HALF_UP);
                        if (subtract.compareTo(warningAmount) > -1 && divide.compareTo(warningProportion) > -1) {
                            sendMessage.sendOnDayMessage(accountId, userAllocation.getAuthName(), maxHour, cost, project.getProjectName(), userAllocation.getUserName());
                        }
                    }
                }
            }

            if (!Check.isNull(nowCost) && nowCost.compareTo(new BigDecimal(0)) != 0) {
                BigDecimal yesterdayHourCost = commonMapper.getCost(accountId, maxHour, anotherDay);
                if (!Check.isNull(yesterdayHourCost) && yesterdayHourCost.compareTo(new BigDecimal(0)) != 0) {
                    BigDecimal subtract = nowCost.subtract(yesterdayHourCost);
                    BigDecimal divide = nowCost.divide(yesterdayHourCost, 2, RoundingMode.HALF_UP);
                    if (subtract.compareTo(warningAmount) > -1 && divide.compareTo(warningProportion) > -1) {
                        sendMessage.sendYesterdayMessage(accountId, userAllocation.getAuthName(), maxHour, cost, project.getProjectName(), userAllocation.getUserName());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
