package cn.com.ctop.job.bytedance.service.impl;

import cn.com.ctop.job.bytedance.entity.WexinConfig;
import cn.com.ctop.job.bytedance.mapper.SendMapper;
import cn.com.ctop.job.bytedance.service.AccountInfoService;
import cn.com.ctop.job.bytedance.service.CommonService;
import cn.com.ctop.job.bytedance.service.IMessageTemplate;
import cn.com.ctop.job.bytedance.service.ISendMessageService;
import cn.com.ctop.job.bytedance.service.ReportDataService;
import cn.com.ctop.job.bytedance.service.TokenService;
import cn.com.ctop.job.bytedance.utils.Check;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;

@Slf4j
@Service
public class SendMessageServiceImpl implements ISendMessageService {
    @Autowired
    private SendMapper sendMapper;

    @Autowired
    private WexinServiceImpl wexinService;

    @Autowired
    private CommonService commonService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private ReportDataService userService;

    @Autowired
    private AccountInfoService accountInfoService;

    @Autowired
    private IMessageTemplate messageTemplate;

    @Override
    public List<WexinConfig> orderByDescList() {
        return sendMapper.orderByDescList();
    }

    @Override
    public void sendYesterdayMessage(Long accountId, String authName, Integer maxHour, BigDecimal cost, String projectName, String userName) {
        String txt = messageTemplate.getYesterdayMessage(accountId, authName, maxHour, cost, projectName, userName);
        List<JSONObject> wexinIds = accountInfoService.selectBytaDanceWeiXinIdListByAccountId(accountId);
        wexinService.sendMessage(wexinIds, txt);

    }

    @Override
    public void sendOnDayMessage(Long accountId, String authName, Integer maxHour, BigDecimal cost, String projectName, String userName) {
        String txt = messageTemplate.getOnDayMessage(accountId, authName, maxHour, cost, projectName, userName);
        List<JSONObject> wexinIds = accountInfoService.selectBytaDanceWeiXinIdListByAccountId(accountId);
        wexinService.sendMessage(wexinIds, txt);
    }

    @Override
    public void sendMessage(String userId, String text) {
        JSONObject weChatId = sendMapper.getWChatIdByUserId(userId);
        if (!Check.isNull(weChatId)) {
            wexinService.sendMessageByWeChatId(weChatId, text);
            log.info("消息发送成功，userId:{}", userId);
        }
    }
}
