package cn.com.ctop.job.bytedance.service.impl;

import cn.com.ctop.job.bytedance.entity.BytedanceAgentToken;
import cn.com.ctop.job.bytedance.entity.CtopOauthToken;
import cn.com.ctop.job.bytedance.mapper.TokenMapper;
import cn.com.ctop.job.bytedance.service.TokenService;
import cn.com.ctop.job.bytedance.utils.Check;
import cn.com.ctop.job.bytedance.utils.HttpUtils;
import com.alibaba.fastjson.JSONObject;
import com.xxl.job.core.context.XxlJobHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class TokenServiceImpl implements TokenService {
    @Autowired
    private TokenMapper tokenMapper;

    @Override
    public List<CtopOauthToken> getAllBytedanceTokens() {
        return tokenMapper.getAllBytedanceTokens();
    }

    @Override
    public List<CtopOauthToken> queryTtValidRuleAccountsToken() {
        return tokenMapper.queryTtValidRuleAccountsToken();
    }

    @Override
    public void refreshBytedanceToken() {
        List<CtopOauthToken> byteDanceOauthTokens = tokenMapper.getAllBytedanceTokens();
        if (!Check.isNull(byteDanceOauthTokens)) {
            for (CtopOauthToken oauthToken : byteDanceOauthTokens) {
                getByteDanceAccessToken(oauthToken);
            }
        }
    }

    public Map<String, Object> getByteDanceAccessToken(CtopOauthToken token) {
        Map<String, Object> resultMap = new HashMap<>();
        String url = "https://ad.oceanengine.com/open_api/oauth2/refresh_token";
        Map<String, Object> param = new HashMap<>();
        param.put("app_id", "1635316529903624");
        param.put("secret", "0c51523e90d6166418fc8421a4896808e90e26f4");
        param.put("grant_type", "refresh_token");
        param.put("refresh_token", token.getRefreshToken());
        String result = HttpUtils.httpPostRequest(url, param, new HashMap<>());
        System.err.println(JSONObject.parseObject(result));
        XxlJobHelper.log("头条刷新token;accountId:{},返回信息：{}", token.getAccountId(), result);
        if (null == result || "".equals(result.trim()) || "null".equalsIgnoreCase(result)) {
            XxlJobHelper.log("头条刷新token失败;accountId:{},返回信息：{}", token.getAccountId(), result);
            resultMap.put("message", "token刷新失败");
            resultMap.put("token", token);
            resultMap.put("code", 0);
            return resultMap;
        }
        JSONObject resultObject = JSONObject.parseObject(result);
        Integer code = resultObject.getInteger("code");
        String message = resultObject.getString("message");
        if (code == 0) {
            JSONObject data = resultObject.getJSONObject("data");
            CtopOauthToken oauthToken = new CtopOauthToken();
            oauthToken.setAccessToken(data.getString("access_token"));
            oauthToken.setRefreshToken(data.getString("refresh_token"));
            Date now = new Date();
            Integer refreshTokenExpiresIn = data.getInteger("refresh_token_expires_in");
            Integer expiresIn = data.getInteger("expires_in");
            oauthToken.setRefreshTokenExpiresIn(addSecond(now, refreshTokenExpiresIn));
            oauthToken.setAccessTokenExpiresIn(addSecond(now, expiresIn));
            oauthToken.setAccountId(oauthToken.getAccountId());
            int i = tokenMapper.updateToken(oauthToken);
            if (i > 0) {
                XxlJobHelper.log("头条刷新token成功:accountId:{}", token.getAccountId());
            }
            resultMap.put("message", "token刷新成功");
            resultMap.put("code", 0);
        } else {
            XxlJobHelper.log("头条刷新token失败;accountId:{},返回信息：{}", token.getAccountId(), message);
            resultMap.put("message", message);
            resultMap.put("token", token);
            resultMap.put("code", 0);
        }
        return resultMap;
    }

    private Date addSecond(Date date, int seconds) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.SECOND, seconds);
        return calendar.getTime();
    }

    @Override
    public void refreshBytedanceAgentToken() {
        List<JSONObject> tokens = tokenMapper.getAgentTokens();
        if (!Check.isNull(tokens)) {
            for (JSONObject token : tokens) {
                refreshAgentBytedanceToken(token);
            }
        }
    }

    public void refreshAgentBytedanceToken(JSONObject token) {
        Map<String, Object> resultMap = new HashMap<>();
        String url = "https://ad.oceanengine.com/open_api/oauth2/refresh_token";
        Map<String, Object> param = new HashMap<>();
        param.put("app_id", "1635316529903624");
        param.put("secret", "0c51523e90d6166418fc8421a4896808e90e26f4");
        param.put("grant_type", "refresh_token");
        param.put("refresh_token", token.getString("refresh_token"));
        String result = HttpUtils.httpPostRequest(url, param, new HashMap<>());
        if (!Check.isNull(result)) {
            JSONObject resultObject = JSONObject.parseObject(result);
            Integer code = resultObject.getInteger("code");
            String message = resultObject.getString("message");
            if (code == 0) {
                JSONObject data = resultObject.getJSONObject("data");
                long accessTokenExpireIn = data.getLong("expires_in") * 1000L;
                long now = System.currentTimeMillis();
                Date accessTokenExpireInDate = new Date(now + accessTokenExpireIn);
                data.put("access_token_expires_in", accessTokenExpireInDate);
                long refreshTokenExpireIn = data.getLong("refresh_token_expires_in") * 1000L;
                Date refreshTokenExpireInDate = new Date(now + refreshTokenExpireIn);
                data.put("refresh_token_expires_in", accessTokenExpireInDate);
                data.put("id", token.getString("id"));
                data.put("agent_id", token.getString("agent_id"));
                data.put("agent_name", token.getString("agent_name"));
                data.put("agent_type", token.getString("agent_type"));
                data.put("app_id", token.getString("app_id"));
                BytedanceAgentToken tokenInfo = JSONObject.parseObject(data.toJSONString(), BytedanceAgentToken.class);
                tokenMapper.replaceAgentToken(tokenInfo);
            }
        }
    }

}
