package cn.com.ctop.job.bytedance.service.impl;

import cn.com.ctop.job.bytedance.entity.WechatAttendance;
import cn.com.ctop.job.bytedance.entity.WechatCheckinData;
import cn.com.ctop.job.bytedance.entity.WechatDepartment;
import cn.com.ctop.job.bytedance.entity.WechatNoList;
import cn.com.ctop.job.bytedance.entity.WechatUserList;
import cn.com.ctop.job.bytedance.enums.DepartEnum;
import cn.com.ctop.job.bytedance.mapper.WechatMapper;
import cn.com.ctop.job.bytedance.service.WechatService;
import cn.com.ctop.job.bytedance.utils.Check;
import cn.com.ctop.job.bytedance.utils.DateUtils;
import cn.com.ctop.job.bytedance.utils.HttpUtils;
import cn.com.ctop.job.bytedance.utils.QYWxConstant;
import cn.com.ctop.job.bytedance.utils.SpNoEnum;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Service
public class WechatServiceImpl implements WechatService {

    @Autowired
    private WechatMapper wechatMapper;

    @Override
    public void getNoListDate(String starttime, String endtime, String dataType) {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            long start = DateUtils.timeToStamp(starttime);
            long end = DateUtils.timeToStamp(endtime);
            String token = getToken(dataType);
            Integer page = 0;
            JSONArray noList = getAllNoList(page, new JSONArray(), start, end, token);
            boolean sleep = false;
            if (noList.size() > 600) {
                sleep = true;
            }
            if (!Check.isNull(noList)) {
                StringBuffer buf = new StringBuffer();
                //获取审批申请详情 URL;
                buf.append(QYWxConstant.APP_ROVAL_DETAIL).append("?access_token=").append(token);
                String userId = null;
                Map<String, String> parameterMap = null;
                for (Object spNo : noList) {
                    parameterMap = new HashMap<>();
                    //接口频率限制 600次/分钟
                    if (sleep) {
                        Thread.sleep(105);
                    }
                    JSONObject params = new JSONObject();
                    params.put("sp_no", spNo);
                    String resultStr = HttpUtils.httpPostRequestTest(buf.toString(), params.toJSONString(), null);
                    JSONObject resultJson = JSONObject.parseObject(resultStr);
                    Integer errcode = resultJson.getInteger("errcode");
                    if (errcode == 0) {
                        JSONObject info = resultJson.getJSONObject("info");
                        String spStatus = info.getString("sp_status");
                        String spName = info.getString("sp_name");
                        /**申请单状态：1-审批中；2-已通过，只记录这两种状态数据
                         * 而且只记录请假、打卡补卡类型
                         **/
                        boolean statusisOK = "2".equals(spStatus) || "1".equals(spStatus);
                        boolean typeisOK = spName.contains("打卡补卡") || spName.contains("请假") || spName.contains("外出") || spName.contains("出差");
                        if (statusisOK && typeisOK) {
                            JSONObject applyer = info.getJSONObject("applyer");
                            if (!Check.isNull(applyer)) {
                                userId = applyer.getString("userid");
                            }
                            //审批申请数据
                            JSONObject applyData = info.getJSONObject("apply_data");
                            //获取审批状态数据
                            getParameterMap(applyData, parameterMap);
                            WechatNoList noEntity = new WechatNoList();
                            noEntity.setUserId(userId);
                            noEntity.setSpNo(spNo.toString());
                            noEntity.setStatus(spStatus);
                            if (!Check.isNull(parameterMap.get("newBegin"))) {
                                noEntity.setNewBegin(DateUtils.stampToTime(Long.valueOf(parameterMap.get("newBegin"))));
                            }
                            if (!Check.isNull(parameterMap.get("newEnd"))) {
                                noEntity.setNewEnd(DateUtils.stampToTime(Long.valueOf(parameterMap.get("newEnd"))));
                            }
                            noEntity.setType(parameterMap.get("type"));
                            noEntity.setNewDuration(parameterMap.get("newDuration"));
                            if ("1".equals(dataType)) {
                                noEntity.setDataType(1);
                            }
                            UUID uuid = UUID.randomUUID();
                            noEntity.setId(uuid.toString());
                            wechatMapper.insertWechatNoList(noEntity);
                        }
                    }
                }
            }
        } catch (Exception e) {
            returnMap.put("code", -1);
            returnMap.put("message", "系统错误");
            log.error("审批异常", e);
        }
    }

    //递归获取审批数据
    private JSONArray getAllNoList(Integer page, JSONArray spNoList, Long starttime, Long endtime, String token) {
        JSONObject data = getNoList(page, starttime, endtime, token);
        spNoList.addAll(data.getJSONArray("sp_no_list"));
        Integer nextCursor = data.getInteger("next_cursor");
        if (!Check.isNull(nextCursor)) {
            return getAllNoList(nextCursor, spNoList, starttime, endtime, token);
        }
        return spNoList;
    }

    /**
     * 批量获取审批单号，单号！
     *
     * @param page      查询起始数，分页查询游标，默认为0
     * @param starttime 审批单提交的时间范围，开始时间，UNix时间戳
     * @param endtime   批单提交的时间范围，结束时间，Unix时间戳
     */
    private JSONObject getNoList(Integer page, Long starttime, Long endtime, String token) {
        try {
            JSONObject params = new JSONObject();
            params.put("starttime", starttime);
            params.put("endtime", endtime);
            params.put("cursor", page);
            params.put("size", 100);
            StringBuffer buf = new StringBuffer();
            //批量获取审批单号 URL
            buf.append(QYWxConstant.APP_ROVAL_INFO).append("?access_token=").append(token);
            String resultStr = HttpUtils.httpPostRequestTest(buf.toString(), params.toJSONString(), null);
            JSONObject resultJson = JSONObject.parseObject(resultStr);
            Integer errcode = resultJson.getInteger("errcode");
            if (errcode == 0) {
                return resultJson;
            } else {
                throw new Exception("批量获取审批单号");
            }
        } catch (Exception e) {
            log.error("批量获取审批单号异常", e);
        }
        return new JSONObject();
    }

    private String getToken(String dataType) throws Exception {
        Map<String, Object> approvaAccessToken = getApprovaAccessToken();
        if ((Integer) approvaAccessToken.get("code") != 0) {
            log.error("企业微信获取token失败,返回信息：{}", approvaAccessToken.get("message"));
            throw new Exception("企业微信获取token失败");
        }
        return approvaAccessToken.get("accessToken").toString();
    }

    public Map<String, Object> getApprovaAccessToken() {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            JSONObject params = new JSONObject();
            params.put("corpid", QYWxConstant.QYWX_CORP_ID);
            params.put("corpsecret", QYWxConstant.QYWX_APPROVA_CORP_SECRET);
            String resultStr = HttpUtils.httpGet(QYWxConstant.QYWX_API_URL + QYWxConstant.QYWX_GET_TOKEN, params, null);
            JSONObject resultJson = JSONObject.parseObject(resultStr);
            Integer errcode = resultJson.getInteger("errcode");
            if (errcode == 0) {
                returnMap.put("code", 0);
                returnMap.put("accessToken", resultJson.getString("access_token"));
            } else {
                returnMap.put("code", errcode);
                returnMap.put("message", resultJson.getString("errmsg"));
            }
        } catch (Exception e) {
            returnMap.put("code", -1);
            returnMap.put("message", "系统错误");
        }
        return returnMap;
    }

    /**
     * 获取审批状态数据
     *
     * @return void
     * @throws
     * @author ZHAOXA
     * @Param [applyData, parameterMap]
     */
    private void getParameterMap(JSONObject applyData, Map<String, String> parameterMap) {
        String control = null;
        if (!Check.isNull(applyData)) {
            //审批申请详情，由多个表单控件及其内容组成
            JSONArray contents = applyData.getJSONArray("contents");
            if (!Check.isNull(contents)) {
                for (int i = 0; i < contents.size(); i++) {
                    //获取content 对象数据
                    JSONObject contentEntity = contents.getJSONObject(i);
                    if (!Check.isNull(contentEntity)) {
                        control = contentEntity.getString("control");
                        //如果补卡组件为PunchCorrection，直接判断为补卡
                        if (SpNoEnum.control3.getCode().equals(control)) {
                            JSONObject value = contentEntity.getJSONObject("value");
                            if (!Check.isNull(value)) {
                                JSONObject punch = value.getJSONObject("punch_correction");
                                if (!Check.isNull(punch)) {
                                    parameterMap.put("type", SpNoEnum.type2.getName());
                                    parameterMap.put("newBegin", punch.getString("time"));
                                    parameterMap.put("newEnd", punch.getString("time"));
                                }
                            }
                            //假勤组件-请假组件 Vacation
                        } else if (SpNoEnum.control2.getCode().equals(control)) {
                            JSONObject value = contentEntity.getJSONObject("value");
                            if (!Check.isNull(value)) {
                                JSONObject vacation = value.getJSONObject("vacation");
                                if (!Check.isNull(vacation)) {
                                    JSONObject selector = vacation.getJSONObject("selector");
                                    if (!Check.isNull(selector)) {
                                        getVacation(selector, parameterMap);
                                    }
                                    JSONObject attendance = vacation.getJSONObject("attendance");
                                    if (!Check.isNull(attendance)) {
                                        getTimeData(attendance, parameterMap);
                                    }
                                }
                            }
                            //假勤组件-出差/外出/加班组件Attendance
                        } else if (SpNoEnum.control1.getCode().equals(control)) {
                            JSONObject value = contentEntity.getJSONObject("value");
                            if (!Check.isNull(value)) {
                                JSONObject attendance = value.getJSONObject("attendance");
                                if (!Check.isNull(attendance)) {
                                    getTimeData(attendance, parameterMap);
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 获取假期类型
     *
     * @param
     * @return void
     * @throws
     * @author ZHAOXA
     */
    private void getVacation(JSONObject selector, Map<String, String> parameterMap) {
        JSONArray options = selector.getJSONArray("options");
        if (!Check.isNull(options)) {
            for (int j = 0; j < options.size(); j++) {
                JSONObject option = options.getJSONObject(j);
                JSONArray values = option.getJSONArray("value");
                if (!Check.isNull(values)) {
                    for (int k = 0; k < values.size(); k++) {
                        JSONObject val = values.getJSONObject(k);
                        parameterMap.put("type", val.getString("text"));
                    }
                }
            }
        }
    }

    /**
     * 获取组件中的假勤类型，假勤开始、结束时间以及时长。
     *
     * @return void
     * @throws
     * @author ZHAOXA
     * @Param [attendance, parameterMap]
     */
    private void getTimeData(JSONObject attendance, Map<String, String> parameterMap) {
        String type = attendance.getString("type");
        //假勤类型，排除类型为1的请假，因为有具体的请假类型，例如：年假，事假等具体类型
        if (!"1".equals(type)) {
            parameterMap.put("type", SpNoEnum.getSpNoEnum(type).getName());
        }
        JSONObject dateRange = attendance.getJSONObject("date_range");
        if (!Check.isNull(dateRange)) {
            parameterMap.put("newBegin", dateRange.getString("new_begin"));
            parameterMap.put("newEnd", dateRange.getString("new_end"));
            BigDecimal duration = dateRange.getBigDecimal("new_duration");
            if (!Check.isNull(duration)) {
                //除以3600，取小时数
                BigDecimal second = new BigDecimal(3600);
                parameterMap.put("newDuration", String.valueOf(duration.divide(second, 1, 4)));
            }
        }
    }


    @Override
    public void getUserList() {
        try {
            //获取token
            Map<String, Object> qywxAccessToken = getQYWXAccessToken();
            if ((Integer) qywxAccessToken.get("code") != 0) {
                log.error("企业微信获取token失败,返回信息：{}", qywxAccessToken.get("message"));
                throw new Exception("企业微信获取token失败");
            }

            JSONObject params = new JSONObject();
            params.put("access_token", qywxAccessToken.get("accessToken"));

            // 更新部门
            getDepartment(params);

            List<WechatDepartment> list = wechatMapper.queryWechatDepartmentList(0);
            if (Check.isNull(list)) {
                return;
            }

            for (WechatDepartment department : list) {
                String departId = department.getDepartId();
                params.put("department_id", departId);
                String resultStr = HttpUtils.httpGet(QYWxConstant.QYWX_USER_SIMPLELIST, params, null);
                JSONObject resultJson = JSONObject.parseObject(resultStr);
                Integer errcode = resultJson.getInteger("errcode");
                if (errcode == 0) {
                    JSONArray userlist = resultJson.getJSONArray("userlist");
                    if (!Check.isNull(userlist)) {
                        for (int i = 0; i < userlist.size(); i++) {
                            JSONObject userJson = userlist.getJSONObject(i);
                            if (!Check.isNull(userJson)) {
                                String userid = userJson.getString("userid");
                                String id = departId + "_" + userid;
                                WechatUserList userList = new WechatUserList();
                                userList.setId(id);
                                userList.setDepartId(departId);
                                userList.setUserId(userid);
                                userList.setUserName(userJson.getString("name"));
                                wechatMapper.insertWechatUserList(userList);
                            }
                        }
                    }
                } else {
                    if (errcode == 60003) {
                        log.error(department.getDepartName() + "，该部门ID不存在");
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void getDepartment(JSONObject params) {
        try {
            Map<String, Object> qywxAccessToken = getQYWXAccessToken();
            if ((Integer) qywxAccessToken.get("code") != 0) {
                log.error("企业微信获取token失败,返回信息：{}", qywxAccessToken.get("message"));
                throw new Exception("企业微信获取token失败");
            }
            String resultStr = HttpUtils.httpGet(QYWxConstant.QYWX_DEPART_LIST, params, null);
            JSONObject resultJson = JSONObject.parseObject(resultStr);
            Integer errcode = resultJson.getInteger("errcode");
            if (errcode == 0) {
                JSONArray department = resultJson.getJSONArray("department");
                if (!Check.isNull(department)) {
                    for (int i = 0; i < department.size(); i++) {
                        JSONObject departJson = department.getJSONObject(i);
                        if (!Check.isNull(departJson)) {
                            Boolean isTrueOrFalse = DepartEnum.getDepart(departJson.getString("name"));
                            if (!isTrueOrFalse) {
                                continue;
                            }
                            wechatMapper.deleteWechatDepartment(departJson.getString("name"));
                            WechatDepartment depart = new WechatDepartment();
                            depart.setDepartId(departJson.getString("id"));
                            depart.setDepartName(departJson.getString("name"));
                            depart.setParentId(departJson.getString("parentid"));
                            depart.setDepartOrder(departJson.getString("order"));
                            depart.setCreateTime(new Date());
                            depart.setUpdateTime(new Date());
                            wechatMapper.insertWechatDepartment(depart);
                        }
                    }
                }
            } else {
                throw new Exception("企业微信获取部门列表");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Map<String, Object> getQYWXAccessToken() {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            TreeMap<String, Object> params = new TreeMap<>();
            params.put("corpid", QYWxConstant.QYWX_CORP_ID);
            params.put("corpsecret", QYWxConstant.QYWX_CORP_SECRET);
            String resultStr = HttpUtils.httpGet(QYWxConstant.QYWX_API_URL + QYWxConstant.QYWX_GET_TOKEN, params, null);
            JSONObject resultJson = JSONObject.parseObject(resultStr);
            Integer errcode = resultJson.getInteger("errcode");
            if (errcode == 0) {
                returnMap.put("code", 0);
                returnMap.put("accessToken", resultJson.getString("access_token"));
            } else {
                returnMap.put("code", errcode);
                returnMap.put("message", resultJson.getString("errmsg"));
            }
        } catch (Exception e) {
            returnMap.put("code", -1);
            returnMap.put("message", "系统错误");
        }
        return returnMap;
    }

    static ExecutorService executorService = Executors.newFixedThreadPool(6);

    @Override
    public void getCheckinData(String starttime, String endtime) {
        try {
            Map<String, Object> qywxAccessToken = getOAAccessToken();
            if ((Integer) qywxAccessToken.get("code") != 0) {
                log.error("员工考勤获取token失败,返回信息：{}", qywxAccessToken.get("message"));
                throw new Exception("员工考勤获取token失败");
            }
            List<WechatUserList> list = wechatMapper.queryWechatUserList(0);
            if (Check.isNull(list)) {
                return;
            }
            Object accessToken = qywxAccessToken.get("accessToken");
            list.forEach(userList -> executorService.submit(() -> addWechatCheckinData(userList, starttime, endtime, accessToken)));
        } catch (Exception e) {
            log.error("员工考勤入库异常，", e);
        }
    }

    private void addWechatCheckinData(WechatUserList userList, String starttime, String endtime, Object accessToken) {
        JSONObject params = new JSONObject();
        String userId = userList.getUserId();
        String departId = userList.getDepartId();
        params.put("opencheckindatatype", 3);
        JSONArray userArr = new JSONArray();
        userArr.add(userId);
        params.put("useridlist", userArr);
        params.put("starttime", DateUtils.timeToStamp(starttime));
        params.put("endtime", DateUtils.timeToStamp(endtime));
        String resultStr = HttpUtils.httpPostRequestTest(QYWxConstant.QYWX_CHECKIN_DATA + "?access_token=" + accessToken, params.toJSONString(), null);
        JSONObject resultJson = JSONObject.parseObject(resultStr);
        if (resultJson.getInteger("errcode") == 0) {
            JSONArray checkindata = resultJson.getJSONArray("checkindata");
            if (!Check.isNull(checkindata)) {
                StringBuffer buf = null;
                String checkinType = "";
                String checkinTypeCode = "";
                for (int i = 0; i < checkindata.size(); i++) {
                    JSONObject jsonObject = checkindata.getJSONObject(i);
                    if (!Check.isNull(jsonObject)) {
                        WechatCheckinData checkinData = new WechatCheckinData();
                        checkinType = jsonObject.getString("checkin_type");
                        //1上班，2下班
                        checkinTypeCode = "上班打卡".equals(checkinType) ? "1" : "2";
                        //下班打卡时间
                        Long checkinTimeStamp = jsonObject.getLong("checkin_time");
                        Date checkDate = DateUtils.stampToTime(checkinTimeStamp);
                        String dateStr = DateUtils.formatDate(checkDate);
                        long dateLong = DateUtils.timeToStamp(dateStr + " 00:00:00");
                        //判断上班是否是周一，周一打卡时间超过9.35则算迟到
                        String exceptionType = jsonObject.getString("exception_type");
                        if ("1".equals(checkinTypeCode)) {
                            if ("星期一".equals(DateUtils.getWhatDay(checkinTimeStamp))) {
                                long toTime = DateUtils.timeToStamp(dateStr + " 09:36:00");
                                if (checkinTimeStamp > toTime) {
                                    exceptionType = Check.isNull(exceptionType) ? "时间异常" : exceptionType;
                                }
                            }
                        }
                        buf = new StringBuffer();
                        String id = buf.append(userId).append("_").append(checkinTypeCode).append("_").append(dateLong).toString();
                        checkinData.setId(id);
                        checkinData.setDepartId(departId);
                        checkinData.setUserId(userId);
                        checkinData.setCheckinType(checkinType);
                        checkinData.setCheckinTime(checkDate);
                        Long dateStamp = DateUtils.timeToStamp(dateStr + " 21:30:00"); // 每天九点半时间戳
                        if (checkinTimeStamp >= dateStamp) {
                            checkinData.setOvertimeOrNot(1);
                            String overtimeStr = DateUtils.formatDate(checkDate) + " 19:00:00";
                            Long overtimeStamp = DateUtils.timeToStamp(overtimeStr); // 每天九点半时间戳
                            Double overtime = Double.valueOf(checkinTimeStamp - overtimeStamp) / 3600;
                            checkinData.setOvertime(overtime);
                        } else {
                            checkinData.setOvertimeOrNot(0);
                        }
                        checkinData.setWifiname(jsonObject.getString("wifiname"));
                        checkinData.setNotes(jsonObject.getString("notes"));
                        checkinData.setLat(jsonObject.getString("lng"));
                        checkinData.setLocationDetail(jsonObject.getString("location_detail"));
                        checkinData.setGroupName(jsonObject.getString("groupname"));
                        checkinData.setDeviceid(jsonObject.getString("deviceid"));
                        checkinData.setExceptionType(exceptionType);
                        checkinData.setLocationTitle(jsonObject.getString("location_title"));
                        checkinData.setWifimac(jsonObject.getString("wifimac"));
                        checkinData.setMediaids(jsonObject.getJSONArray("mediaids").toJSONString());
                        checkinData.setLat(jsonObject.getString("lat"));
                        wechatMapper.insertWechatCheckinData(checkinData);
                    }
                }
            }
        } else {
            log.error("获取打卡数据异常,返回数据-> {} ；入参-> {}", resultJson, params.toJSONString());
        }
    }

    public Map<String, Object> getOAAccessToken() {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            TreeMap<String, Object> params = new TreeMap<>();
            params.put("corpid", QYWxConstant.QYWX_CORP_ID);
            params.put("corpsecret", QYWxConstant.QYWX_OA_CORP_SECRET);
            String resultStr = HttpUtils.httpGet(QYWxConstant.QYWX_API_URL + QYWxConstant.QYWX_GET_TOKEN, params, null);
            JSONObject resultJson = JSONObject.parseObject(resultStr);
            Integer errcode = resultJson.getInteger("errcode");
            if (errcode == 0) {
                returnMap.put("code", 0);
                returnMap.put("accessToken", resultJson.getString("access_token"));
            } else {
                returnMap.put("code", errcode);
                returnMap.put("message", resultJson.getString("errmsg"));
            }
        } catch (Exception e) {
            returnMap.put("code", -1);
            returnMap.put("message", "系统错误");
        }
        return returnMap;
    }

    @Override
    public List<WechatAttendance> queryAttendanceData(Map<String, Object> paramsMap) {
        finishingTime(paramsMap);
        List<WechatAttendance> list = wechatMapper.queryAttendanceData(paramsMap);
        if (!Check.isNull(list)) {
            checkOverTimeData(list);
        }
        return list;
    }

    /**
     * 整理时间
     */
    private void finishingTime(Map<String, Object> paramsMap) {
        String startTime = (String) paramsMap.get("startTime");
        if (!Check.isNull(startTime)) {
            startTime += " 00:00:00";
            paramsMap.put("startTime", startTime);
        }
        String stopTime = (String) paramsMap.get("stopTime");
        if (!Check.isNull(stopTime)) {
            stopTime += " 23:59:59";
            paramsMap.put("stopTime", stopTime);
        }
    }

    /**
     * 筛选10:30以后打卡的迟到数据，判断前一天是否加班，更新迟到时长
     */
    private void checkOverTimeData(List<WechatAttendance> list) {
        for (WechatAttendance attendance : list) {
            if ("迟到".equals(attendance.getType())) {
                Integer dur = Integer.valueOf(attendance.getStateDuration().replace(" min", ""));
                if (dur - 60 >= 0) {
                    String checkinDate = attendance.getCheckinDate();
                    String lastDay = DateUtils.getLastDay(checkinDate, 1);
                    Map<String, Object> map = new HashMap<>();
                    map.put("userId", attendance.getUserId());
                    map.put("checkinDate", lastDay);
                    map.put("checkinType", "下班打卡");
                    WechatCheckinData data = wechatMapper.queryEntityByMap(map);
                    if (!Check.isNull(data) && "1".equals(data.getOvertimeOrNot() + "")) {
                        attendance.setStateDuration((dur - 60) + " min");
                    }
                }
            }
        }
    }

    @Override
    public List<WechatAttendance> queryOverTimeData(Map<String, Object> paramsMap) {
        finishingTime(paramsMap);
        return wechatMapper.queryOverTimeData(paramsMap);
    }
}
